package sun.net.www.protocol.http;

import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.util.Iterator;
import java.util.Set;
import java.util.Collections;
import java.util.HashMap;
import sun.net.www.HeaderParser;
import sun.net.www.MessageHeader;

public class AuthenticationHeader
{
    MessageHeader rsp;
    HeaderParser preferred;
    String preferred_r;
    private final HttpCallerInfo hci;
    boolean dontUseNegotiate;
    static String authPref;
    String hdrname;
    HashMap<String, AuthenticationHeader.SchemeMapValue> schemes;
    
    @Override
    public String toString() {
        return "AuthenticationHeader: prefer " + this.preferred_r;
    }
    
    public AuthenticationHeader(String s, MessageHeader messageHeader, HttpCallerInfo httpCallerInfo, boolean b) {
        this(s, messageHeader, httpCallerInfo, b, Collections.emptySet());
    }
    
    public AuthenticationHeader(String hdrname, MessageHeader rsp, HttpCallerInfo hci, boolean dontUseNegotiate, Set<String> set) {
        this.dontUseNegotiate = false;
        this.hci = hci;
        this.dontUseNegotiate = dontUseNegotiate;
        this.rsp = rsp;
        this.hdrname = hdrname;
        this.schemes = new HashMap();
        this.parse(set);
    }
    
    public HttpCallerInfo getHttpCallerInfo() {
        return this.hci;
    }
    
    private void parse(Set<String> set) {
        Iterator<String> multiValueIterator = this.rsp.multiValueIterator(this.hdrname);
        while (multiValueIterator.hasNext()) {
            String raw = (String)multiValueIterator.next();
            HeaderParser o = new HeaderParser(raw);
            Iterator<String> keys = o.keys();
            int end = 0;
            int n = -1;
            while (keys.hasNext()) {
                keys.next();
                if (o.findValue(end) == null) {
                    if (n != -1) {
                        HeaderParser subsequence = o.subsequence(n, end);
                        String key = subsequence.findKey(0);
                        if (!set.contains(key)) {
                            this.schemes.put(key, new AuthenticationHeader.SchemeMapValue(subsequence, raw));
                        }
                    }
                    n = end;
                }
                ++end;
            }
            if (end > n) {
                HeaderParser subsequence2 = o.subsequence(n, end);
                String key2 = subsequence2.findKey(0);
                if (!set.contains(key2)) {
                    this.schemes.put(key2, new AuthenticationHeader.SchemeMapValue(subsequence2, raw));
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        AuthenticationHeader.SchemeMapValue schemeMapValue = null;
        if (AuthenticationHeader.authPref == null || (schemeMapValue = (AuthenticationHeader.SchemeMapValue)this.schemes.get(AuthenticationHeader.authPref)) == null) {
            if (schemeMapValue == null && !this.dontUseNegotiate) {
                AuthenticationHeader.SchemeMapValue schemeMapValue2 = (AuthenticationHeader.SchemeMapValue)this.schemes.get("negotiate");
                if (schemeMapValue2 != null) {
                    if (this.hci == null || !NegotiateAuthentication.isSupported(new HttpCallerInfo(this.hci, "Negotiate"))) {
                        schemeMapValue2 = null;
                    }
                    schemeMapValue = schemeMapValue2;
                }
            }
            if (schemeMapValue == null && !this.dontUseNegotiate) {
                AuthenticationHeader.SchemeMapValue schemeMapValue3 = (AuthenticationHeader.SchemeMapValue)this.schemes.get("kerberos");
                if (schemeMapValue3 != null) {
                    if (this.hci == null || !NegotiateAuthentication.isSupported(new HttpCallerInfo(this.hci, "Kerberos"))) {
                        schemeMapValue3 = null;
                    }
                    schemeMapValue = schemeMapValue3;
                }
            }
            if (schemeMapValue == null && (schemeMapValue = (AuthenticationHeader.SchemeMapValue)this.schemes.get("digest")) == null && (!NTLMAuthenticationProxy.supported || (schemeMapValue = (AuthenticationHeader.SchemeMapValue)this.schemes.get("ntlm")) == null)) {
                schemeMapValue = (AuthenticationHeader.SchemeMapValue)this.schemes.get("basic");
            }
        }
        else if (this.dontUseNegotiate && AuthenticationHeader.authPref.equals("negotiate")) {
            schemeMapValue = null;
        }
        if (schemeMapValue != null) {
            this.preferred = schemeMapValue.parser;
            this.preferred_r = schemeMapValue.raw;
        }
    }
    
    public HeaderParser headerParser() {
        return this.preferred;
    }
    
    public String scheme() {
        if (this.preferred != null) {
            return this.preferred.findKey(0);
        }
        return null;
    }
    
    public String raw() {
        return this.preferred_r;
    }
    
    public boolean isPresent() {
        return this.preferred != null;
    }
    
    static {
        AuthenticationHeader.authPref = null;
        AuthenticationHeader.authPref = (String)AccessController.doPrivileged(new GetPropertyAction("http.auth.preference"));
        if (AuthenticationHeader.authPref != null) {
            AuthenticationHeader.authPref = AuthenticationHeader.authPref.toLowerCase();
            if (AuthenticationHeader.authPref.equals("spnego") || AuthenticationHeader.authPref.equals("kerberos")) {
                AuthenticationHeader.authPref = "negotiate";
            }
        }
    }
    
    static class SchemeMapValue
    {
        String raw;
        HeaderParser parser;
        
        SchemeMapValue(HeaderParser parser, String raw) {
            this.raw = raw;
            this.parser = parser;
        }
    }
}
